#' predictLNC
#'
#' This function launches the 10 coding potential prediction processes using the 10 tools, the tables generated will be saved in the folder "tmp". The ten tools include: CPAT, CPC2, LGC, LncADeep, LncDC, lncFinder, longdist, mRNN, RNAminingm and RNASamba  
#' @param fastafile: the path to the fasta file containing the nucleotide sequence(s) of the set of the transcripts to be predicted.
#' @param config_file: the path to the config.ini file containing all the paths to the binary and executable files of the 10 coding potential tools included in our meta-tool and their own python environment.
#' @param blastp_pfam: Boolean, should blastp against swissprot and pfam domains be searched for similarity (Default is FALSE), Note that blastp and pfam runs are time and memory consuming.
#' @param out_file: the name of the output file (without any extension)
#' @export
#' @keywords coding potential prediction
#' @examples
#' nt_file=system.file("extdata", "test_nt.fa", package = "votingLNC")
#' get the config file
#' config=system.file("extdata", "config.ini", package = "votingLNC") #please adapt the paths in the this file according to your installation (refer to the configr package how to build a config.ini file)
#' or build a config.ini file with the same structure as the config.ini included with the package.
#' predictLNC(fastafile=nt_file, config_file=config, out_file= "out_file", blastp_pfam=FALSE)
#' or by including blastp and pfam filters
#' predictLNC(fastafile=nt_file, config_file=config, out_file= "out_file", blastp_pfam=TRUE)
predictLNC <- function(fastafile=NULL, config_file=NULL, out_file= NULL, blastp_pfam=FALSE){
	
	#config_file=system.file('config.ini', package = 'votingLNC', mustWork=TRUE)
	#ini=configr::read.config(file = config_file)
	ini=configr::read.config(file = config_file)
	#retrieve the python configurations (multiple envs)
	py3_env=ini$py3$path
	py2_env=ini$py2$path
	lncdc_env=ini$lncdc$path
	rnamining_env=ini$rnamining$path
	#retrieve the excecutable file location
	ezLncPred=ini$ezLncPred$path
	LncADeep=ini$LncADeep$path
	LncDC=ini$LncDC$path
	RNAmining=ini$RNAmining$path
	mRNN=ini$mRNN$path
	RNASamba=ini$RNASamba$path
	mRNN_models=ini$mRNN_models$path
	rnasamba_models=ini$RNASamba_models$path
	#blastp and pfam variables
	blastp_db_idx=ini$swissprot_idx$path
	pfam_hmm=ini$pfam_hmm$path
	
	#Clean the tmp folder if exists, or create a new folder otherwise
	if(dir.exists(file.path(".", "tmp"))){
		unlink(file.path(".", "tmp"), recursive=TRUE)
	}
	dir.create(file.path(".", "tmp"))
	
	
	#check the content of the fasta file, and clean it if applicable
	dir.create(file.path(".", "tmp", "nt_file"))
	cleanFasta(ntfastafile=fastafile, outdir=file.path(".", "tmp", "nt_file"), outfile=out_file)
	fastafile=file.path(".", "tmp", "nt_file",out_file)
	
	#launch CPC2
	dir.create(file.path(".", "tmp", "CPC2"))
	system(paste0(py3_env," ",ezLncPred, " -i ", fastafile, " -o ", 
				  file.path(".", "tmp", "CPC2", out_file), " CPC2"), 
				  wait=TRUE, ignore.stdout = TRUE, ignore.stderr = TRUE)
	
	#launch CPAT
	dir.create(file.path(".", "tmp", "CPAT"))
	system(paste0(py3_env," ",ezLncPred, " -i ", fastafile, " -o ", 
				  file.path(".", "tmp", "CPAT", out_file), " CPAT -p Human"), 
				  wait=TRUE, ignore.stdout = TRUE, ignore.stderr = TRUE)
	
	#launch LGC
	dir.create(file.path(".", "tmp", "LGC"))
	system(paste0(py3_env," ",ezLncPred, " -i ", fastafile, " -o ", 
				  file.path(".", "tmp", "LGC", out_file), " lgc"), 
				  wait=TRUE, ignore.stdout = TRUE, ignore.stderr = TRUE)
	
	#launch longdist
	dir.create(file.path(".", "tmp", "longdist"))
	system(paste0(py3_env," ",ezLncPred, " -i ", fastafile, " -o ", 
				  file.path(".", "tmp", "longdist", out_file), " longdist -z 0"), 
				  wait=TRUE, ignore.stdout = TRUE, ignore.stderr = TRUE)

	#LncADeep
	dir.create(file.path(".", "tmp", "LncADeep"))
	keras::use_backend("theano")
	system(paste0(py2_env," ",LncADeep, " -MODE lncRNA -f ", 
				fastafile, " -o ", out_file), 
				wait=TRUE, ignore.stdout = TRUE, ignore.stderr = TRUE)
	system(paste0("mv ", file.path(paste0(out_file,"_LncADeep_lncRNA_results"),paste0(out_file,"_LncADeep.results"))," ",file.path(".", "tmp", "LncADeep", out_file)), 
				wait=TRUE)
	unlink(file.path(".", paste0(out_file,"_LncADeep_lncRNA_results")), recursive=TRUE)
	
	#LncDC
	dir.create(file.path(".", "tmp", "LncDC"))
	system(paste0(lncdc_env," ",LncDC, " -i ", fastafile, " -o ", 
				  file.path(".", "tmp", "LncDC", out_file), " -t 4"), 
				  wait=TRUE, ignore.stdout = TRUE, ignore.stderr = TRUE)
	
	#mRNN
	dir.create(file.path(".", "tmp", "mRNN"))
	mrnn_model_1=paste0(mRNN_models,"w10u5.pkl")
	mrnn_model_2=paste0(mRNN_models,"w14u3.pkl")
	mrnn_model_3=paste0(mRNN_models,"w16u5.pkl")
	mrnn_model_4=paste0(mRNN_models,"w18u1b.pkl")
	mrnn_model_5=paste0(mRNN_models,"w23u2.pkl")
	system(paste0(py2_env," ",mRNN, " -w ", mrnn_model_1,",",
											mrnn_model_2,",",
											mrnn_model_3,",",
											mrnn_model_4,",",
											mrnn_model_5, " -o ", file.path(".", "tmp", "mRNN", out_file)," ", fastafile), 
											ignore.stdout = TRUE, ignore.stderr = TRUE, wait=TRUE)


	#RNAmining: the output folder should be created first
	dir.create(file.path(".", "tmp", "RNAmining"))
	system(paste0(rnamining_env," ",RNAmining, " -f ", fastafile, " -organism_name Homo_sapiens -prediction_type coding_prediction -output_folder ", 
				  file.path(".", "tmp", "RNAmining")), 
				   wait=TRUE, ignore.stdout = TRUE, ignore.stderr = TRUE)
	system(paste0("mv ", file.path(".","tmp","RNAmining","predictions.txt")," ", file.path(".", "tmp", "RNAmining", out_file)), 
				wait=TRUE)

	#RNASamba: change the engine from theano to tensorflow
	dir.create(file.path(".", "tmp", "RNASamba"))
	fullmodel_samba=paste0(rnasamba_models,"full_length_weights.hdf5")
	partialmodel_samba=paste0(rnasamba_models,"partial_length_weights.hdf5")
	keras::use_backend("tensorflow")
	system(paste0(py3_env," ",RNASamba," ",file.path(".", "tmp", "RNASamba", out_file), " ", 
				  fastafile, " ", fullmodel_samba, " ", fullmodel_samba), 
				  wait=TRUE, ignore.stdout = TRUE, ignore.stderr = TRUE)
	
	#LncFinder (R)
	dir.create(file.path(".", "tmp", "LncFinder"))
	seq=seqinr::read.fasta(fastafile)
	tab=LncFinder::lnc_finder(Sequences=seq, SS.features=FALSE, format="DNA", frequencies.file="human", svm.model="human", parallel.cores = 4)
	write.csv(tab, file=file.path(".", "tmp", "LncFinder", out_file))
	
	#blastp and pfam search (if applicable)
	if(blastp_pfam){
		dir.create(file.path(".", "tmp", "blastp"))
		dir.create(file.path(".", "tmp", "pfam"))
		dir.create(file.path(".", "tmp", "peptide_files"))
		#generate peptide files
		getPeptideFile(ntfastafile=fastafile, outdir=file.path(".", "tmp", "peptide_files/"), outfile=out_file)
		pepfastafile=file.path(".", "tmp", "peptide_files",out_file)
		#blastp
		blastp(query=pepfastafile, dbidx=blastp_db_idx, evalue=1e-5, max_hits=10, output=file.path(".", "tmp", "blastp/"), outfile=out_file)
		#pfam
		pfam(query=pepfastafile, refdb=pfam_hmm, E=0.001, domE=0.05, output=file.path(".", "tmp", "pfam/"), outfile=out_file, cpu=8)
	}
}

#' votingLNC
#'
#' This main function implements the majority voting pipeline which combines the prediction results of 10 coding potential tools. The final class is determined by the majority voting procedure. This 
#' function quantifies the content of the transcriptome (percentage of coding, lncRNA, and sncRNA), and classifies the predicted lncRNAs according to their reliability/confidence level. Furthermore,
#' it generates two plots summarizing the results
#' @param fastafile: the path to the fasta file containing the nucleotide sequence(s) of the set of the transcripts to be predicted.
#' @param prob: a numeric value indicating the threshold of the non-coding potential probability of a transcript to be classified as lncRNA by the majority voting procedure used here.
#' @param tr.len: a numeric value (nt) indicating the threshold of the length of a transcript to be classified as lncRNA. LncRNA are arbitrary defined as a sequence greater than 200 nt (default value).
#' @param pep.len: a numeric value (nt) indicating the threshold of the peptide length of a transcript (if applicable) to be classified as lncRNA. Default is assigned to 100 nt corresponding to ~ 33 aa  
#' @param blastp_pfam: a boolean, should blastp against swissprot and pfam domains be searched for similarity (Default is FALSE), Note that blastp and pfam runs are time and memory consuming, but increase the reliability.
#' @param plot: a boolean indicating whether the results of the prediction will be plotted or not
#' @export
#' @keywords coding potential prediction, majority voting, lncRNA confidence
#' @examples
#' nt_file=system.file("extdata", "test_nt.fa", package = "votingLNC")
#' votingLNC(fastafile=nt_file, config_file="config.ini", prob=0.5, tr.len=200, pep.len=100, blastp_pfam=FALSE, plot=TRUE)
#' including blastp and pfam search, preferably to run in a cluster 
#' votingLNC(fastafile=nt_file, config_file="config.ini", prob=0.5, tr.len=200, pep.len=100, blastp_pfam=FALSE, plot=TRUE)
votingLNC <- function(fastafile=NULL, config_file=NULL, prob=0.5, tr.len=200, pep.len=100, blastp_pfam=FALSE, plot=TRUE){
	tools=c("CPC2", "CPAT", "LGC", "LncADeep", "LncDC", "LncFinder", "longdist", "mRNN", "RNAmining", "RNASamba")
	#system('cat logo.txt')
	logo_file=file.path(.libPaths()[1], "votingLNC", "others", "logo.txt")
	#logo_file="./inst/others/logo.txt"	
	system(paste0("cat ", logo_file))
	cat(paste0("\n INFO 1: Predicting the coding potential of the sequences including in ", fastafile," ...\n"))
	#generate a random tmp filename
	out_file <- basename(tempfile())
	predictLNC(fastafile=fastafile, config_file=config_file, out_file=out_file, blastp_pfam=blastp_pfam)	
	cat("INFO 2: Process all the tables coming from each tool ...\n")
	pathFolder="./tmp"
	
	list.df=pbapply::pblapply(1:length(tools), function(i){
		filepath=file.path(".", "tmp", tools[i], out_file)
		tab=readTableTool(tools[i], filepath=filepath)
		
		if(i==2){ #CPAT
			rownames(tab)=unlist(lapply(1:nrow(tab), function(p){
					strsplit(rownames(tab)[p], "[']")[[1]][2]
			}))
		}
		
		if(i==5){
			filepath=file.path(".", "tmp", tools[1], out_file)
			refLabels=rownames(readTableTool(tools[1], filepath=filepath))
			labels=unlist(lapply(1:length(refLabels), function(p){
				ifelse(refLabels[p] %in% rownames(tab),
						   tab[rownames(tab)==refLabels[p],,drop=F]$class,
						   NA)
			}))
			tab=data.frame(class=labels, row.names=refLabels)
		}
		rownames(tab)=toupper(rownames(tab))
		tab
	})
	
	cat("INFO 3: collecting and ordering the dataframes ...\n")
	pred.df.tmp=do.call(cbind, pbapply::pblapply(1:length(list.df), function(j){
			tab.ref=list.df[[1]]
			tab.target=list.df[[j]]
			#convert rownames to Uppercase
			rownames(tab.ref)=toupper(rownames(tab.ref))
			rownames(tab.target)=toupper(rownames(tab.target))
			tab.target=tab.target[match(rownames(tab.ref), rownames(tab.target)),,drop=F]
			colnames(tab.target)[ncol(tab.target)]=paste0("Class.",tools[j])
			tab.target
	}))
	
	
	pred.df.tmp=pred.df.tmp[,c(2:3, 9:ncol(pred.df.tmp)),drop=F]
	
	cat("INFO 4: Calculate the majority voted classes ...\n")
	maj.classes=unlist(pbapply::pblapply(1:nrow(pred.df.tmp), function(p){
			x=as.numeric(pred.df.tmp[p,3:ncol(pred.df.tmp)])
			x<-x[!is.na(x)]
			as.numeric(names(which.max(table(x))))
	}))

	cat("INFO 5: Calculate the probabilities ...\n")
	probabilities=unlist(pbapply::pblapply(1:nrow(pred.df.tmp), function(p){
			x=as.numeric(pred.df.tmp[p,3:ncol(pred.df.tmp)])
			x<-x[!is.na(x)]
			as.numeric(decimal((length(x[x==1])/length(x)),2))
	}))
	
	pred.df.tmp$maj.class=maj.classes
	pred.df.tmp$probability=probabilities
	
	pred.df.tmp=pred.df.tmp[,c(1,2,13,14),drop=F]
	`%>%` <- magrittr::`%>%`
	if(!blastp_pfam){
		df.majority <- pred.df.tmp %>%
						dplyr::mutate(Majority_Class = dplyr::case_when(maj.class==1 & transcript_length > tr.len & peptide_length < pep.len & probability >= prob ~ "lncRNA-like",
																		maj.class==1 & transcript_length <= tr.len & peptide_length < pep.len & probability >= prob ~ "sncRNA-like",
												 TRUE ~ "Coding-like"))
	}else{
		cat("INFO 6: Queering SwissProt for similarity and Pfam for homology ...\n")
		#read the result from blastp and pfam
		df.blastp=readblastp(file.path(".","tmp","blastp",out_file))
		df.blastp$qseqid=toupper(df.blastp$qseqid)
		#filtering: All hits with an E-value < 1e-6 and identity (pident) >=98% were considered as significant blast hits.
		#for each tanscript, if a significant hit was reported then display it otherwise "NA". For transcripts with more than one hit, report only the one with the highest similarity.
		significant.blastp = do.call(rbind, pbapply::pblapply(1: nrow(pred.df.tmp), function(p){
				current.df = df.blastp[df.blastp$qseqid == rownames(pred.df.tmp)[p],,drop=F]
				current.df = current.df[current.df$evalue < 1e-5 & current.df$pident >=90,,drop=F]
				if(nrow(current.df)==0){
						data.frame(best.hit.blastp = "NA", e.value.blastp=1, similarity.blastp = 0, row.names=rownames(pred.df.tmp)[p])
				}else{
						best.hit.idx=which.max(current.df$pident)
						best.hit.df=current.df[best.hit.idx,,drop=F]
						data.frame(best.hit.blastp = best.hit.df$sseqid,
										   e.value.blastp= best.hit.df$evalue,
										   similarity.blastp = best.hit.df$pident,
										   row.names=rownames(pred.df.tmp)[p])
				}
		}))
		
		#read pfam results
		df.pfam=micropan::readHmmer(hmmer.file=file.path(".","tmp","pfam",out_file))
		df.pfam$Query=toupper(df.pfam$Query)
		
		#for each tanscript, if a significant hit was reported then display it otherwise "NA". For transcripts with more than one hit, report only the one with the highest similarity.
		significant.pfam = do.call(rbind, pbapply::pblapply(1: nrow(pred.df.tmp), function(p){
				current.df = df.pfam[df.pfam$Query == rownames(pred.df.tmp)[p],,drop=F]
				current.df = current.df[current.df$Evalue < 0.001,,drop=F]
				if(nrow(current.df)==0){
						data.frame(best.hit.pfam = "NA", e.value.pfam = 1, function.pfam="NA", row.names=rownames(pred.df.tmp)[p])
				}else{
						best.hit.idx=which.min(current.df$Evalue)
						best.hit.df=current.df[best.hit.idx,,drop=F]
						data.frame(best.hit.pfam = strsplit(best.hit.df$Hit, "[.]")[[1]][1],
										   e.value.pfam = best.hit.df$Evalue,
										   function.pfam = best.hit.df$Description,
										   row.names=rownames(pred.df.tmp)[p])
				}
		}))
		pred.df.tmp=data.frame(pred.df.tmp, 
							   significant.blastp, 
							   significant.pfam)
		df.majority <- pred.df.tmp %>%
						dplyr::mutate(Majority_Class = dplyr::case_when(maj.class==1 & transcript_length > tr.len & peptide_length < pep.len & probability >= prob & similarity.blastp < 90 & e.value.pfam > 0.001 ~ "lncRNA-like",
														  maj.class==1 & transcript_length <= tr.len & peptide_length < pep.len & probability >= prob & similarity.blastp < 90 & e.value.pfam > 0.001 ~ "sncRNA-like",
												 TRUE ~ "Coding-like"))
		
	}
	coding=length(which(df.majority$Majority_Class=="Coding-like"))
	lncrna=length(which(df.majority$Majority_Class=="lncRNA-like"))
	sncrna=length(which(df.majority$Majority_Class=="sncRNA-like"))
	ncrna=lncrna+sncrna
	#plotting pichart transcriptomics content
	if(plot){
		data.piechart=data.frame(Class=c("Coding-like","lncRNA-like","sncRNA-like"),
						  value=c(decimal(coding/nrow(df.majority),2), decimal(lncrna/nrow(df.majority),2),decimal(sncrna/nrow(df.majority),2)))
		plot1=piechart.plot(data.piechart)
		#plot reliability of lncRNAs found
		df.confidence <- df.majority[df.majority$Majority_Class=="lncRNA-like",,drop=F] %>%
							dplyr::mutate(Confidence = dplyr::case_when(Majority_Class=="lncRNA-like" & probability ==1 ~ "High-confidence",
														  Majority_Class=="lncRNA-like" & probability >= 0.8 & probability < 1 ~ "Good-confidence",
														  Majority_Class=="lncRNA-like" & probability > 0.5 & probability < 0.8 ~ "Medium-confidence",
														  TRUE ~ "-"))
		hclncrna=length(which(df.confidence$Confidence=="High-confidence"))
		gclncrna=length(which(df.confidence$Confidence=="Good-confidence"))
		mclncrna=length(which(df.confidence$Confidence=="Medium-confidence"))
		
		data.barplot=data.frame(Species=rep("species", 3),
								Class=c("Good.confidence","High.confidence","Medium.confidence"),
								value=c(decimal(gclncrna/nrow(df.confidence),2), decimal(hclncrna/nrow(df.confidence),2),decimal(mclncrna/nrow(df.confidence),2)))
		
		data.barplot$Class<-factor(data.barplot$Class, levels = c("Medium.confidence","Good.confidence","High.confidence"))	
		plot2=bar.plot(data.barplot)				  
		#put together the two plots and save
		pdf("transcriptomics_content.pdf", width = 16, height = 8)
		gridExtra::grid.arrange(plot1, plot2, ncol=2, nrow=1)
		dev.off()
	}
	cat("..............Summary .....................\n")
	cat("...........................................\n")
	cat("...........................................\n")
	cat("Parameters:\n")
	cat(paste0("Input file: ", fastafile,"\n"))
	cat(paste0("Probability threshold: ", prob,"\n"))
	cat(paste0("Transcript length: ", tr.len, "\n"))
	cat(paste0("Peptide length: ", pep.len, "\n"))
	cat("\n")
	cat("Results:\n")
	cat(paste0("-------ncRNAs: ", ncrna,"(",decimal(ncrna*100/nrow(df.majority),2),"%)","\n"))
	cat(paste0("-------------lncRNA-like: ", lncrna,"(",decimal(lncrna*100/ncrna,2),"%)","\n"))
	cat(paste0("-------------sncRNA-like: ", sncrna,"(",decimal(sncrna*100/ncrna,2),"%)","\n"))
	cat(paste0("-------Coding: ", coding, "(",decimal(coding*100/nrow(df.majority),2),"%)", "\n"))
	#delete the temp files/folders
	if(dir.exists(file.path(".", "tmp"))){
		unlink(file.path(".", "tmp"), recursive=TRUE)
	}
	write.csv(df.majority, file="predictions.csv")
	sink("summary.txt")
	cat("\t.............. votingLNC prediction summary .....................\n")
	cat("\t..................................................................\n")
	cat("\t..................................................................\n")
	cat("Parameters:\n")
	cat(paste0("Input file: ", fastafile,"\n"))
	cat(paste0("Probability threshold: ", prob,"\n"))
	cat(paste0("Transcript length: ", tr.len, "\n"))
	cat(paste0("Peptide length: ", pep.len, "\n"))
	cat("\n")
	cat("Results:\n")
	cat(paste0("-------ncRNAs: ", ncrna,"(",decimal(ncrna*100/nrow(df.majority),2),"%)","\n"))
	cat(paste0("-------------lncRNA-like: ", lncrna,"(",decimal(lncrna*100/ncrna,2),"%)","\n"))
	cat(paste0("-------------sncRNA-like: ", sncrna,"(",decimal(sncrna*100/ncrna,2),"%)","\n"))
	cat(paste0("-------Coding: ", coding, "(",decimal(coding*100/nrow(df.majority),2),"%)", "\n"))
	sink()
	
}

#' cleanFasta
#'
#' This function cleans the non-ACGT DNA alphabet chars from the input nucleatide sequences 
#' @param ntfastafile: the path to the fasta file containing the nucleotide sequences
#' @param outdir: the output folder 
#' @export
#' @keywords cleaning fasta file 
#' @examples
#' nt_file=system.file("extdata", "test_nt.fa", package = "votingLNC")
#' cleanFasta(ntfastafile=nt_file, outdir="./output", outfile="nt_file_cleaned.fa")

cleanFasta <- function(ntfastafile, outdir=NULL, outfile=NULL){
        
		seq=seqinr::read.fasta(ntfastafile)
        seq.replaced = lapply(1:length(seq), function(i){
                sequence = seq[[i]]
                sequence[which(!sequence%in%c("a","c","g","t","n"))]="n"
                toupper(sequence)
        })
		#extract headers
		
        seqinr::write.fasta(seq.replaced, names=names(seq), file.out=file.path(outdir, outfile))
}

#' getPeptideFile
#'
#' This function generates a fasta file containing the peptide sequences of the nucleotide sequences provided in the input file
#' @param ntfastafile: the path to the fasta file containing the nucleotide sequences
#' @param outdir: the output folder 
#' @export
#' @keywords ORF, peptide 
#' @examples
#' nt_file=system.file("extdata", "test_nt.fa", package = "votingLNC")
#' getPeptideFile(ntfastafile=nt_file, outdir="./output", outfile="test_pep.fa")
getPeptideFile <- function(ntfastafile, outdir=NULL, outfile=NULL){
        
        orf_max = ftrCOOL::maxORF(seqs=ntfastafile, reverse=FALSE)
        pep_seqs=lapply(1:length(orf_max), function(i){
                pep.seq=seqinr::translate(seqinr::s2c(orf_max[[i]]), frame = 0, sens = "F", numcode = 1, NAstring = "X", ambiguous = TRUE)
                pep.seq=paste(pep.seq, collapse="")
                pep.seq=strsplit(pep.seq, "[*]")[[1]]
                list.length=unlist(lapply(pep.seq, nchar))
                pep.seq[which(list.length==max(list.length))]
        })
        seqinr::write.fasta(pep_seqs, names=names(orf_max), file.out=file.path(outdir,outfile))
}


#' blastp
#'
#' This function is a wrapper for the NCBI Protein-Protein BLAST against Swissprot database only
#' @param query: Query sequences as path to a FASTA file containing peptide sequence(s)
#' @param dbidx: Path to the folder containing the indexes of the database to BLAST against (please include the suffix)
#' @param evalue: Expect value cutoff (default: 10)
#' @param max_hits: How many hits per sequence to return (default: 20)
#' @param ouput: the folder the blastp result will be saved inside
#' @export
#' @keywords blastp, NCBI, Swissprot
#' @examples
#' pep_file=system.file("extdata", "test_pep.fa", package = "votingLNC")
#' path_to_db_indexes="..." 
#' blastp(query=pep_file, dbidx=path_to_db_indexes, evalue=0.005, max_hits=10, output="./output/", outfile="test")
blastp <- function(query=NULL, dbidx=NULL, evalue=0.005, max_hits=10, output=NULL, outfile=NULL){
	system(paste0("blastp -query ",query," -db ",dbidx," -out ", output, outfile,
				  " -evalue ", evalue, " -max_target_seqs ", max_hits, " -num_threads  8 -outfmt 6"), 
				  wait=TRUE)
}

#' pfam
#'
#' This function is a wrapper for hmmscan function to perform a HMMER search against the Pfam database 
#' @param query: Query sequences as path to a FASTA file containing peptide sequence(s)
#' @param refdb: Path to the pfam database to be used as reference
#' @param E: the E-value cutoff for models to be reported 
#' @param domE: the E-value cutoff for protein domains to be reported 
#' @param ouput: the folder the pfam result table will be saved inside
#' @param cpu: number of computing cpus to be used 
#' @export
#' @keywords protein domain, Pfam
#' @examples
#' pep_file=system.file("extdata", "test_pep.fa", package = "votingLNC")
#' path_to_hmm_model=".../Pfam-A.hmm"
#' pfam(query=pep_file, refdb=path_to_hmm_model, E=0.001, domE=0.05, output="./output/", outfile="test", cpu=4)
pfam <- function(query=NULL, refdb=NULL, E=0.001, domE=0.05, output=NULL, outfile, cpu=1){
	system(paste0("hmmscan --cpu ", cpu," --domtblout ", output, outfile," -E ", 
				   E," --domE ",domE," --noali ", refdb,"  ",query," > ",output,"pfam_result.log"), 
				  wait=TRUE)
}


#' readblastp
#'
#' This function load and process the table generated by blastp
#' @param filepath: absolute path to the table generated by blastp
#' @export
#' @keywords read blastp
#' @examples
#' 
#' readblastp("./output/test.blastp")
readblastp <- function(filepath){
        colnames.blastp=c("qseqid","sseqid","pident","length","mismatch","gapopen","qstart","qend","sstart","send","evalue","bitscore")
		#first check if the file is empty
		if(!file.size(filepath)==0){
				df.blastp=read.table(filepath, sep='\t', header=F)
		}else{ # otherwise, create an empty df
				df.blastp = data.frame(matrix(nrow = 0, ncol = length(colnames.blastp)))
		}
		colnames(df.blastp)=colnames.blastp
		df.blastp$qseqid=toupper(df.blastp$qseqid)
		return(df.blastp)
}


#' readTableTool
#'
#' This function load the output results coming from each coding potential prediction tool: CPAT, CPC2, LGC, LncADeep, LncDC, lncFinder, longdist, mRNN, RNAmining, RNASamba
#' @param tool: the name of the tool
#' @param filepath: the path to the temp folder containing the tables to load, subfolders must have the same name as the tool names.
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="CPC2"
#' readTableTool(tool, "./data/")
readTableTool <- function(tool, filepath){
	switch(tool,
			"CPC2" = {
					tab = readCPC2(filepath) #used as a reference
			},
			"CPAT" = {
					tab = readCPAT(filepath)
			},
			"LGC" = {
					tab = readLGC(filepath)
			},
			"LncADeep" = {
					tab = readLncADeep(filepath)
			},
			"LncDC" = {
					tab = readLncDC(filepath)
			},
			"LncFinder" = {
					tab = readLncFinder(filepath)
			},
			"longdist" = {
					tab = readLongdist(filepath)
			},
			"mRNN" = {
					tab = readmRNN(filepath)
			},
			"RNAmining" = {
					tab = readRNAmining(filepath)
			},
			"RNASamba" = {
					tab = readRNAsamba(filepath)
			},
			{
			   stop("No corresponding read for your choice !", call.=FALSE)
			})
	return(tab)
}		
		
#' readCPC2
#'
#' This function load and process the table generated by CPC2
#' @param filepath: absolute path to the table generated by CPC2
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="CPC2"
#' readTableTool(tool, ".data/")
readCPC2 <- function(filepath){
        tab=read.table(filepath)
        colnames(tab)=c("ID","transcript_length","peptide_length","Fickett_score","pI","ORF_integrity","coding_probability","label")
        tab$class=ifelse(tab$label=="noncoding",1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$ID),]
        rownames(tab)=tab$ID

        return(tab)
}

#' readCPAT
#'
#' This function load and process the table generated by CPAT
#' @param filepath: absolute path to the table generated by CPAT
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="CPAT"
#' readTableTool(tool, ".data/")
readCPAT <- function(filepath){
        tab=read.table(filepath, row.names=NULL)
        tab$class=ifelse(tab$coding_prob<0.5,1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$row.names),]
        rownames(tab)=tab$row.names
        tab$row.names=NULL
        return(tab[,c(6),drop=F])
}

#' readLGC
#'
#' This function load and process the table generated by LGC
#' @param filepath: absolute path to the table generated by LGC
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="LGC"
#' readTableTool(tool, ".data/")
readLGC <- function(filepath){
        tab=read.table(filepath)
        tab$class=ifelse(tab$V5=="Non-coding",1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$V1),]
        rownames(tab)=tab$V1
        return(tab[,c(10),drop=F])
}

#' readLncADeep
#'
#' This function load and process the table generated by LncADeep
#' @param filepath: absolute path to the table generated by LncADeep
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="LncADeep"
#' readTableTool(tool, ".data/")
readLncADeep <- function(filepath){
        tab=read.table(filepath, header=F, skip=1)
        tab$class=ifelse(tab$V3=="Noncoding",1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$V1),]
        rownames(tab)=tab$V1
        return(tab[,c(25),drop=F])
}

#' readLncDC
#'
#' This function load and process the table generated by LncDC
#' @param filepath: absolute path to the table generated by LncDC
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="LncDC"
#' readTableTool(tool, ".data/")
readLncDC <- function(filepath, refLabels){
        tab=read.csv(filepath, header=T)
        tab$class=ifelse(tab$predict=="lncrna",1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$Description),]
        rownames(tab)=tab$Description
		return(tab[,c(27),drop=F])
}

#' readLncFinder
#'
#' This function load and process the table generated by LncFinder
#' @param filepath: absolute path to the table generated by LncFinder
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="LncFinder"
#' readTableTool(tool, ".data/")
readLncFinder <- function(filepath){
        tab=read.csv(filepath, header=T)
        tab$class=ifelse(tab$Pred=="NonCoding",1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$X),]
        rownames(tab)=tab$X
        return(tab[,ncol(tab),drop=F])
}

#' readLongdist
#'
#' This function load and process the table generated by Longdist. Longdist produces a dataframe with truncated rownames of the transcripts, but in the same order as CPC2 (using the option -z 0 longdist predict all the transcripts)
#' @param filepath: absolute path to the table generated by Longdist
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="Longdist"
#' readTableTool(tool, ".data/")
readLongdist <- function(filepath){
        tab=read.csv(filepath, header=T)
        colnames(tab)=c("ID","prob.pct","prob.lncRNA")
        tab$class=ifelse(tab$prob.lncRNA > 0.5,1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$ID),]

        rownames(tab)=tab$ID
        return(tab[,c(4),drop=F])
}

#' readmRNN
#'
#' This function load and process the table generated by mRNN
#' @param filepath: absolute path to the table generated by mRNN
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="mRNN"
#' readTableTool(tool, ".data/")
readmRNN <- function(filepath){
        tab=read.table(filepath, header=F)
        tab$class=ifelse(tab$V2 > 0.5,0,1)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$V1),]
        rownames(tab)=tab$V1
        return(tab[,c(4),drop=F])
}

#' readRNAmining
#'
#' This function load and process the table generated by RNAmining
#' @param filepath: absolute path to the table generated by RNAmining
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="RNAmining"
#' readTableTool(tool, ".data/")
readRNAmining <- function(filepath){
        tab=read.table(filepath, header=F, skip=5)
        tab$class=ifelse(tab$V2 == "non-coding",1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$V1),]
        rownames(tab)=tab$V1
        return(tab[,c(4),drop=F])
}

#' readRNAsamba
#'
#' This function load and process the table generated by RNARNAsamba
#' @param filepath: absolute path to the table generated by RNARNAsamba
#' @export
#' @keywords load table coding potential tool
#' @examples
#' tool="RNARNAsamba"
#' readTableTool(tool, ".data/")
readRNAsamba <- function(filepath){
        tab=read.table(filepath, header=T)
        tab$class=ifelse(tab$classification == "noncoding",1,0)
        #Remove duplicated trancripts
        tab=tab[!duplicated(tab$sequence_name),]
        rownames(tab)=tab$sequence_name
        return(tab[,c(4),drop=F])
}

#' piechart.plot
#'
#' This function plots the result of the majority voting classification of the transcriptomics content using a piechart plot
#' @param data: a dataframe containing the percentage of the predicted classes
#' @export
#' @keywords piechart plot
#' @examples
#' data=data.frame(Class=c("Coding-like", "lncRNA-like", "sncRNA-like"),
#'                 value=c(0.43, 0.39, 0.18))
#' piechart.plot(data)
piechart.plot <- function(data){
		
		`%>%` <- magrittr::`%>%`
		colors=c(t_col(wesanderson::wes_palette("Zissou1", n = 5, type="discrete")[4], percent = 50),
		     t_col(wesanderson::wes_palette("Zissou1", n = 5, type="discrete")[5], percent = 50),
		     t_col(wesanderson::wes_palette("Zissou1", n = 5, type="discrete")[2], percent = 50))
		cols=c(colors[3], colors[2], colors[1])
		data$value=as.numeric(data$value)*100
		
		dat <- data %>%
		  dplyr::arrange(dplyr::desc(Class)) %>%
		  dplyr::mutate(lab.ypos = cumsum(value) - 0.5 * value)
		
		dat$Class=as.factor(dat$Class)
		
		p <- ggplot2::ggplot(dat, ggplot2::aes(x = 2, y = value, fill = Class)) +
		  ggplot2::geom_bar(stat = "identity", color = "white") +
		  ggplot2::coord_polar(theta = "y", start = 0)+
		  ggplot2::ggtitle("Transcriptomics content") +
		  ggrepel::geom_label_repel(data = dat,
								   ggplot2::aes(y = lab.ypos, label = paste0(format(round(value, 2), nsmall = 2), "%")),
								   size = 4.5, nudge_x = 1, show.legend = FALSE)  +
		  ggplot2::guides(fill = ggplot2::guide_legend(title = "Class")) +
		  ggplot2::scale_fill_manual(values = cols) +
		  ggplot2::theme_void()+
		  ggplot2::xlim(0.5, 2.5) +
		  ggplot2::theme(plot.title = ggplot2::element_text(size=20, face="bold", hjust=0.5, color="grey30"),
						legend.text=ggplot2::element_text(color="grey30"))
	  return(p)
}

								   
#' bar.plot
#'
#' This function plots the confidence level of the transcripts predicted as lncRNAs by the majority voting procedure
#' @param data: a dataframe containing the percentage of the predicted lncRNAs subclasses
#' @export
#' @keywords bar plot
#' @examples
#' data=data.frame(Species=rep("sp",3),
#'                 Class=c("Good.confidence", "High.confidence", "Medium.confidence"),
#'                 value=c(0.33, 0.60, 0.07))
#' bar.plot(data)
bar.plot <- function(data){	

		colors=c(t_col(wesanderson::wes_palette("Zissou1", n = 5, type="discrete")[5], percent = 70),
					 t_col(wesanderson::wes_palette("Zissou1", n = 5, type="discrete")[5], percent = 45),
					 t_col(wesanderson::wes_palette("Zissou1", n = 5, type="discrete")[5], percent = 20))
		data$value=as.numeric(data$value)			 
		p <- ggplot2::ggplot(data, ggplot2::aes(fill=Class, y=value, x=Species)) +
			ggplot2::geom_bar(position="stack", stat="identity") +
			ggplot2::scale_fill_manual(values = colors) +
			ggplot2::scale_y_continuous(labels = scales::percent) +
			ggplot2::geom_text(ggplot2::aes(label = paste0(decimal(value*100,2),"%")),
				position = ggplot2::position_stack(vjust = 0.5), size = 5) +
			ggplot2::ggtitle("LncRNA confidence level") +	
			ggthemes::theme_hc() +
			ggplot2::theme(
						axis.text.x=ggplot2::element_blank(),
						axis.ticks.x=ggplot2::element_blank(),
						axis.ticks.y=ggplot2::element_blank(),
						axis.title.x=ggplot2::element_blank(),
						axis.title.y=ggplot2::element_blank(),
						axis.text.y = ggplot2::element_blank())
		return(p)				
}				

#' t_col
#'
#' This function generates a transparent color
#' @param color: color name (code)
#' @param percent: % transparency
#' @param name: an optional name for the color
#' @export
#' @keywords color transparency RGBA
#' @examples
#' t_col("#E1AF007F", percent=30, name="color1")				
t_col <- function(color, percent = 50, name = NULL) {
	 
	## Get RGB values for named color
	rgb.val <- col2rgb(color)

	## Make new color using input color as base and alpha set by transparency
	t.col <- rgb(rgb.val[1], rgb.val[2], rgb.val[3],
				 max = 255,
				 alpha = (100 - percent) * 255 / 100,
				 names = name)

	## Save the color
	invisible(t.col)
}

#' decimal
#'
#' This function rounds a number of digits of a number
#' @param x: numeric
#' @param n: number of digits to display after the comma
#' @export
#' @keywords decimal, round
#' @examples
#' decimal(0.222334, n=2)
#' output: "0.22"
decimal <- function(x, n=NULL){
        format(round(x, n), nsmall = n)
}